/* 
 * $Id: vscan-fprot_core.c,v 1.0 2002/03/11 15:45:24 reniar Exp $	
 *
 * Core Interface for F-Prot Antivirus Daemon			
 *
 * Copyright (C) Rainer Link, SuSE GmbH <link@suse.de>, 2001-2002
 *                            OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * Credits to W. Richard Stevens - RIP
 * 
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include <includes.h>
#include "vscan-fprotd_core.h"
 

/* read a line from descriptor - reads byte-by-byte -> very slow! */
/* code from UNP by WRS */
/* FIXME !!! */

ssize_t readline(int fd, void *vptr, size_t maxlen) 
{
        ssize_t n, rc;
        char c, *ptr;

        ptr = vptr;
        for (n = 1; n < maxlen; n++) {
           again:
                if ( (rc = read(fd, &c, 1)) == 1 ) {
                        *ptr++ = c;
                        if ( c == '\n' )
                                break;
                } else if ( rc == 0 ) {
                        if ( n == 1 )
                                return (0);
                        else
                                break;
                } else {
                        if ( errno == EINTR )
                                goto again;
                        return (-1);
                }
        }
        *ptr = 0;

        return (n);

}


/* read n bytes from descriptor
   from UNP from WRS
*/

ssize_t readn(int fd, void *vptr, size_t n)
{

	size_t nleft;
	ssize_t nread;
	char *ptr;

	ptr = vptr;
	nleft = n;
	while ( nleft > 0 ) {
		if ( (nread = read(fd, ptr, nleft)) < 0 ) {
			if ( errno == EINTR )
				nread = 0;	/* cand call read() again */
			else
				return (-1);
		} else if ( nread == 0 )
			break;			/* EOF */

		nleft -= nread;
		ptr += nread;
	}

	return (n - nleft);			/* return >= 0 */

}



/* write n bytes to descriptor
   from UNP from WRS
*/

ssize_t writen(int fd, const void *vptr, size_t n)
{
	size_t nleft;
	ssize_t nwritten;
	const char *ptr;

	ptr = vptr;
	nleft = n;
	while ( nleft > 0 ) {
		if ( (nwritten = write(fd, ptr, nleft)) <= 0 ) {
			if ( errno == EINTR )
				nwritten = 0;	/* and call write() again */
			else
				return (-1);	/* error */
			
		}
		nleft -= nwritten;
		ptr += nwritten;
	}

	return (n);

}


/* print a message via syslog */
void vscan_fprotd_print(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY, "%s", printMsg);
}



/* initialise socket to F-Prot Daemon 
   returns -1 on error or the socket descriptor */
int vscan_fprotd_init(void)
{

	int sockfd;
	struct sockaddr_in servaddr;

	/* create socket */
        if (( sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) {
               vscan_fprotd_print("ERROR: can not create socket!\n");
               return -1; 
        }

	bzero(&servaddr, sizeof(servaddr));
        servaddr.sin_family = AF_INET;
        servaddr.sin_port = htons(FPROTD_PORT);

	/* hm, inet_pton may not exist on all systems - FIXME ! */
        if ( inet_pton(AF_INET, FPROTD_IP, &servaddr.sin_addr) <= 0 ) {
                vscan_fprotd_print("ERROR: inet_pton failed!\n");
                return -1;
	}

	/* connect to socket */
	if ( connect(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 )
        {
                vscan_fprotd_print("ERROR: can not connect to F-Prot Daemon!\n");
                return -1;
        }


	return sockfd;

}

/*
void vscan_fprotd_syslog(char *syslogMessage, ...)
{
        char logMsg[512];

        va_list argptr;
        va_start(argptr, syslogMessage);
        vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s\n", logMsg);
}

*/

/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_fprotd_log_virus(char *infected_file, char *result)
{
	char *str;

	/* remove "<name>" and "</name>"from the result string to get only the virus name - hack alert ;) */
	str = result;
	str+= 6;
	str[strlen(str)-8] = '\0';

        vscan_fprotd_print("Scan result: '%s' infected with virus '%s' - Access denied", infected_file, str);
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
  Expects socket descriptor and file name to scan for
  Return -1 on error, 0 if no virus was found, 1 if a virus was found 
*/
int vscan_fprotd_scanfile(int sockfd, char *scan_file)
{
	int n;
	size_t len;
	char recvline[MAXLINE + 1];
	pstring fprotdCommand;
	char *str;


        #if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
        vscan_fprotd_print("Scanning file : '%s'", scan_file);
        #endif

	/* F-Prot Daemon expects "GET <filename>[?<arguments>] HTTP/1.0\n\n" */
	/* what about if the <filename> itself contains '\n'? */
        fstrcpy(fprotdCommand, "GET ");
        fstrcat(fprotdCommand, scan_file);
	fstrcat(fprotdCommand, "?");
	fstrcat(fprotdCommand, FPROTD_ARGS);
        fstrcat(fprotdCommand, " HTTP/1.0\n\n");

	/* write to socket */
	/* NOTE: what happens if scan_file is very long? */
	len = strlen(fprotdCommand);
	if ( len < 0 ) {
		/* overflow? */
		vscan_fprotd_print("ERROR: file name is too long!");
		return -1;
	}	

	n = writen(sockfd, fprotdCommand, len);
	if ( n <= 0 ) {
		vscan_fprotd_print("ERROR: can not send file name to F-Prot Daemon!");
		return -1;
	}

	/* read from socket, line by line */
	while ( (n = readline(sockfd, recvline, MAXLINE)) > 0 ) {
		if ( n < 0 ) {
			vscan_fprotd_print("ERROR: can not get result from F-Prot Daemon!");
			return -1;
		}

		/* ignore the HTTP response header, remove any leading 
		   white spaces */
		str = NULL;
		str = strchr(recvline, '<');
		if ( str != NULL ) {
			if ( strncmp(str, "<name>", 6) == 0 ) {
				/* virus found */
				vscan_fprotd_log_virus(scan_file, str);
				return 1;
			} else if ( strncmp(str, "<error>", 7) == 0 ) {
				/* ERROR */
				#if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
				vscan_fprotd_print("file %s not found or not readable", scan_file);
				#endif
			}
		}
	}

 	/* OK */
	#if VSCAN_FPROTD_VERBOSE_FILE_LOGGING==1
        vscan_fprotd_print("file %s is clean", scan_file);
        #endif


	return 0;
	
}

/*
  close socket
*/
void vscan_fprotd_end(int sockfd)
{
	/* sockfd == -1 indicates an error while connecting to socket */
	if ( sockfd >= 0 ) {
		close(sockfd);
	}

}
